@@ -80,9 +80,9 @@ static zend_object_value php_sphinx_client_new(zend_class_entry *ce TSRMLS_DC) /
 	c = ecalloc(1, sizeof(*c));
 	zend_object_std_init(&c->std, ce TSRMLS_CC);
 
+#if PHP_VERSION_ID < 50399
 	ALLOC_HASHTABLE(c->std.properties);
 	zend_hash_init(c->std.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
-#if PHP_VERSION_ID < 50399
 	zend_hash_copy(c->std.properties, &ce->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));
 #else
 	object_properties_init(&c->std, ce);
@@ -99,13 +99,10 @@ static zval *php_sphinx_client_read_property(zval *object, zval *member, int typ
 static zval *php_sphinx_client_read_property(zval *object, zval *member, int type TSRMLS_DC) /* {{{ */
 #endif
 {
-	php_sphinx_client *c;
 	zval tmp_member;
 	zval *retval;
 	zend_object_handlers *std_hnd;
 
-	c = (php_sphinx_client *)zend_object_store_get_object(object TSRMLS_CC);
-
 	if (member->type != IS_STRING) {
 		tmp_member = *member;
 		zval_copy_ctor(&tmp_member);
@@ -134,18 +131,21 @@ static HashTable *php_sphinx_client_get_properties(zval *object TSRMLS_DC) /* {{
 	php_sphinx_client *c;
 	const char *warning, *error;
 	zval *tmp;
+	HashTable *props;
 
 	c = (php_sphinx_client *)zend_objects_get_address(object TSRMLS_CC);
 
+	props = zend_std_get_properties(object TSRMLS_CC);
+
 	error = sphinx_error(c->sphinx);
 	MAKE_STD_ZVAL(tmp);
 	ZVAL_STRING(tmp, (char *)error, 1);
-	zend_hash_update(c->std.properties, "error", sizeof("error"), (void *)&tmp, sizeof(zval *), NULL);
+	zend_hash_update(props, "error", sizeof("error"), (void *)&tmp, sizeof(zval *), NULL);
 
 	warning = sphinx_warning(c->sphinx);
 	MAKE_STD_ZVAL(tmp);
 	ZVAL_STRING(tmp, (char *)warning, 1);
-	zend_hash_update(c->std.properties, "warning", sizeof("warning"), (void *)&tmp, sizeof(zval *), NULL);
+	zend_hash_update(props, "warning", sizeof("warning"), (void *)&tmp, sizeof(zval *), NULL);
 	return c->std.properties;
 }
 /* }}} */
@@ -605,6 +605,31 @@ static PHP_METHOD(SphinxClient, setFilter)
 }
 /* }}} */
 
+#ifdef HAVE_SPHINX_ADD_FILTER_STRING
+/* {{{ proto bool SphinxClient::setFilterString(string attribute, string value[, bool exclude]) */
+static PHP_METHOD(SphinxClient, setFilterString)
+{
+	php_sphinx_client *c;
+	char *attribute, *value;
+	int	attribute_len, value_len, res;
+	zend_bool exclude = 0;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|b", &attribute, &attribute_len, &value, &value_len, &exclude) == FAILURE) {
+		return;
+	}
+
+	c = (php_sphinx_client *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	SPHINX_INITIALIZED(c)
+
+	res = sphinx_add_filter_string(c->sphinx, attribute, value, exclude ? 1 : 0);
+	if (!res) {
+		RETURN_FALSE;
+	}
+	RETURN_TRUE;
+}
+/* }}} */
+#endif
+
 /* {{{ proto bool SphinxClient::setFilterRange(string attribute, int min, int max[, bool exclude]) */
 static PHP_METHOD(SphinxClient, setFilterRange)
 {
@@ -1550,6 +1575,10 @@ static PHP_METHOD(SphinxClient, escapeString)
 			case '&':
 			case '/':
 			case '\\':
+			case '^':
+			case '$':
+			case '=':
+			case '<':
 				*target++ = '\\';
 				*target++ = *source;
 				break;
@@ -1785,6 +1814,12 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_sphinxclient_setfilter, 0, 0, 2)
 	ZEND_ARG_INFO(0, exclude)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_sphinxclient_setfilterstring, 0, 0, 2)
+	ZEND_ARG_INFO(0, attribute)
+	ZEND_ARG_INFO(0, value)
+	ZEND_ARG_INFO(0, exclude)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_sphinxclient_setfilterrange, 0, 0, 3)
 	ZEND_ARG_INFO(0, attribute)
 	ZEND_ARG_INFO(0, min)
@@ -1906,6 +1941,9 @@ static zend_function_entry sphinx_client_methods[] = { /* {{{ */
 	PHP_ME(SphinxClient, setConnectTimeout, 	arginfo_sphinxclient_setconnecttimeout, ZEND_ACC_PUBLIC)
 	PHP_ME(SphinxClient, setFieldWeights, 		arginfo_sphinxclient_setindexweights, ZEND_ACC_PUBLIC)
 	PHP_ME(SphinxClient, setFilter, 			arginfo_sphinxclient_setfilter, ZEND_ACC_PUBLIC)
+#ifdef HAVE_SPHINX_ADD_FILTER_STRING
+	PHP_ME(SphinxClient, setFilterString, 		arginfo_sphinxclient_setfilterstring, ZEND_ACC_PUBLIC)
+#endif
 	PHP_ME(SphinxClient, setFilterFloatRange, 	arginfo_sphinxclient_setfilterrange, ZEND_ACC_PUBLIC)
 	PHP_ME(SphinxClient, setFilterRange, 		arginfo_sphinxclient_setfilterrange, ZEND_ACC_PUBLIC)
 	PHP_ME(SphinxClient, setGeoAnchor, 			arginfo_sphinxclient_setgeoanchor, ZEND_ACC_PUBLIC)
